SystemOrganization addCategory: #'ExifTool-Core'!

Object subclass: #ETExif
	instanceVariableNames: 'file properties pathContainingExifTool'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExifTool-Core'!

!ETExif commentStamp: 'FrancoisStephany 11/24/2011 21:21' prior: 0!
This package wraps the perl library 'ExifTool'. http://www.sno.phy.queensu.ca/~phil/exiftool/

Common Scenario:
exif := ETExif new file: (FSLocator imageDirectory / 'example.jpg') resolve.
exif run. "will retrieve all the infos about the given picture"
exif properties. "All the infos are stored in a dictionary. Inspect it for more details"

"e.g., to get the keywords of a picture: "
exif properties at: #keywords.

If your exif tool installation is not in the path of OSProcess, you can add it:

exif addPathContainingExifTool: '/opt/bin/'!

!ETExif methodsFor: 'accessing' stamp: 'FrancoisStephany 11/24/2011 21:21'!
addPathContainingExifTool: aString
	self pathContainingExifTool add: aString.! !

!ETExif methodsFor: 'private' stamp: 'FrancoisStephany 11/30/2011 16:32'!
buildCommand
	" complete reference: http://www.sno.phy.queensu.ca/~phil/exiftool/exiftool_pod.html
	-j json output
	 -E escapes HTML characters
	-L uses latin1
	-w EXT : save in a file with the given extension
	-struct does not seem to work for Keywords 
		(i.e., we get a string and not an array when theres only one keyword)"
	^ 'exiftool -struct -j -w txt ' , self escapedFullName! !

!ETExif methodsFor: 'private' stamp: 'FrancoisStephany 11/24/2011 21:16'!
buildOSProcessExtraPath
	^':' join: self pathContainingExifTool! !

!ETExif methodsFor: 'default' stamp: 'FrancoisStephany 11/24/2011 21:05'!
defaultPathContaintingExifTool
	^OrderedCollection with: '/usr/local/bin/'! !

!ETExif methodsFor: 'executing' stamp: 'FrancoisStephany 11/24/2011 16:56'!
deleteOutputFile
	self outputFile delete.! !

!ETExif methodsFor: 'accessing' stamp: 'FrancoisStephany 11/28/2011 13:50'!
escapedFullName
	" convert space into \[space] "
	^self file fullName copyWithRegex: ' ' matchesReplacedWith: '\ '
! !

!ETExif methodsFor: 'accessing' stamp: 'FrancoisStephany 11/19/2011 13:00'!
file
	^ file! !

!ETExif methodsFor: 'accessing' stamp: 'FrancoisStephany 11/19/2011 13:00'!
file: anObject
	file := anObject! !

!ETExif methodsFor: 'private' stamp: 'FrancoisStephany 12/22/2011 16:25'!
fixKeywords
	" hack to deal with an ExiftTool bug: -struct does not work for keywords.
	 so if an image contains only one keyword it will appear as a single value not as
	 a collection of one element.
	We also check that all the keywords are strings. Because the JSon package will return a number
	if one of the keyword is a number. We want all string."
	| keywords |
	keywords := self properties at: #keywords ifAbsent: [ OrderedCollection new ].
	
	keywords := (keywords isString or: [keywords isNumber])
		ifTrue: [ OrderedCollection with: keywords greaseString ]
		ifFalse: [ keywords collect: [:each | each greaseString ]].
		
	self properties at: #keywords put: keywords.! !

!ETExif methodsFor: 'accessing' stamp: 'FrancoisStephany 11/24/2011 17:12'!
outputFile
	^self file parent / (self file base , '.txt')! !

!ETExif methodsFor: 'accessing' stamp: 'FrancoisStephany 11/24/2011 21:20'!
pathContainingExifTool
	^pathContainingExifTool ifNil: [pathContainingExifTool :=  self defaultPathContaintingExifTool]! !

!ETExif methodsFor: 'accessing' stamp: 'FrancoisStephany 11/24/2011 18:04'!
properties
	^ properties ifNil: [properties := Dictionary new]! !

!ETExif methodsFor: 'accessing' stamp: 'FrancoisStephany 11/19/2011 13:17'!
properties: anObject
	properties := anObject! !

!ETExif methodsFor: 'executing' stamp: 'FrancoisStephany 12/22/2011 16:20'!
readPropertiesFromFile:  aFileReference
	| jsonString byteArray capitalizedProperties keywords |
	byteArray := aFileReference readStream contents.
	jsonString := GRPharoUtf8Codec new decode: byteArray.
	capitalizedProperties := (Json readFrom: jsonString readStream) first.
	capitalizedProperties keysAndValuesDo: [:key :value |
		self properties 
			at: key asLegalSelector asSymbol 
			put: value ].
	self fixKeywords.! !

!ETExif methodsFor: 'executing' stamp: 'FrancoisStephany 1/19/2012 20:50'!
run
	| resultString path env pipe |
	self file isFile ifFalse: [ self error: 'You must specify the picture file' ].
	
	env := OSProcess thisOSProcess environment copy.
	path := (env at: #PATH), ':' , self buildOSProcessExtraPath.
	env at: #PATH put: path.
	
	self file exists
		ifFalse: [ self error: 'The file does not exist'].
	
	pipe := PipeableOSProcess command: self buildCommand environment: env.
	resultString := pipe output.
	pipe closePipes.
	"we need to wait the end of the execution of the command"
	self outputFile exists 
		ifTrue: [
			self readPropertiesFromFile: self outputFile.
			self deleteOutputFile ]
		ifFalse: [ self error: 'Error while generating exif file.' , resultString ]! !
